@@ -22,6 +22,31 @@
 #include "php_imagick_defs.h"
 #include "php_imagick_macros.h"
 
+MagickBooleanType php_imagick_progress_monitor(const char *text, const MagickOffsetType offset, const MagickSizeType span, void *client_data)
+{
+	FILE *fp;
+	php_imagick_object *intern = (php_imagick_object *)client_data;
+	
+	if (!intern) {
+		return MagickFalse;
+	}
+	
+	if (!intern->progress_monitor_name) {
+		return MagickFalse;
+	}
+
+	fp = fopen(intern->progress_monitor_name, "a+");
+	
+	if (!fp) {
+		return MagickFalse;
+	}
+
+	fprintf(fp, "text: %s, offset: %ld, span: %ld\n", text, offset, span);
+	fclose(fp);
+	return MagickTrue;	
+}
+
+
 int count_occurences_of(char needle, char *haystack TSRMLS_DC)
 {
 	int occurances = 0;
@@ -102,21 +127,17 @@ double *get_double_array_from_zval(zval *param_array, long *num_elements TSRMLS_
 	for (i = 0 ; i < elements ; i++) {
 
 		if (zend_hash_get_current_data(ht, (void**)&ppzval) == FAILURE) {
-			efree(ht);
+			efree(double_array);
 			double_array = (double *)NULL;
 			return double_array;
 		}
 
 		if(Z_TYPE_PP(ppzval) == IS_LONG) {
 			double_array[i] = (double)Z_LVAL_PP(ppzval);
-
 		} else if (Z_TYPE_PP(ppzval) == IS_DOUBLE) {
-
 			double_array[i] = Z_DVAL_PP(ppzval);
-		
 		} else {
-
-			efree(ht);
+			efree(double_array);
 			double_array = (double *)NULL;
 			return double_array;
 		}
@@ -152,9 +173,20 @@ int read_image_into_magickwand(php_imagick_object *intern, char *filename, int t
 {
 	int error = IMAGICK_READ_WRITE_NO_ERROR;
 	MagickBooleanType status;
-	char *absolute = expand_filepath(filename, NULL TSRMLS_CC);
+	char *absolute;
 
-	if (strlen(filename) > MAXPATHLEN) {
+	if (!filename) {
+		return IMAGICK_READ_WRITE_NO_ERROR;
+	}
+
+	absolute = expand_filepath(filename, NULL TSRMLS_CC);
+
+	if (!absolute) {
+		return IMAGICK_READ_WRITE_NO_ERROR;
+	}
+
+	if (strlen(absolute) > MAXPATHLEN) {
+		efree(absolute);
 		return IMAGICK_READ_WRITE_FILENAME_TOO_LONG;
 	}
 
@@ -249,15 +281,12 @@ zend_bool crop_thumbnail_image(MagickWand *magick_wand, long desired_width, long
 	}
 	
 	if (((double)orig_width / (double)desired_width) > ((double)orig_height / (double)desired_height)) {
-
 		ratio = (double)orig_height / (double)desired_height;
 		image_width = (double)orig_width / (double)ratio;
 		image_height = desired_height;
 		
 		crop_x = ((double)image_width - (double)desired_width) / 2;
-
 	} else {
-
 		ratio = (double)orig_width / (double)desired_width;
 		image_height = (double)orig_height / (double)ratio;
 		image_width = desired_width;
@@ -293,7 +322,7 @@ void deallocate_wands(MagickWand *magick, DrawingWand *draw, PixelWand *pixel TS
 
 void *get_pointinfo_array(zval *coordinate_array, int *num_elements TSRMLS_DC)
 {
-    PointInfo *coordinates;
+	PointInfo *coordinates;
 	long elements, sub_elements, i;
 	HashTable *coords;
 	zval **ppzval, **ppz_x, **ppz_y;
@@ -318,6 +347,7 @@ void *get_pointinfo_array(zval *coordinate_array, int *num_elements TSRMLS_DC)
 		/* Get the sub array */
 		if (zend_hash_get_current_data(coords, (void**)&ppzval) == FAILURE) {
 			coordinates = (PointInfo *)NULL;
+			efree(coordinates);
 			*num_elements = 0;
 			return coordinates;
 		}
@@ -325,6 +355,7 @@ void *get_pointinfo_array(zval *coordinate_array, int *num_elements TSRMLS_DC)
 		/* If its something than array lets error here */
 		if(Z_TYPE_PP(ppzval) != IS_ARRAY) {
 			coordinates = (PointInfo *)NULL;
+			efree(coordinates);
 			*num_elements = 0;
 			return coordinates;
 		}
@@ -335,6 +366,7 @@ void *get_pointinfo_array(zval *coordinate_array, int *num_elements TSRMLS_DC)
 		/* Exactly two elements */
 		if (sub_elements != 2) {
 			*num_elements = 0;
+			efree(coordinates);
 			coordinates = (PointInfo *)NULL;
 			return coordinates;
 		}
@@ -344,12 +376,14 @@ void *get_pointinfo_array(zval *coordinate_array, int *num_elements TSRMLS_DC)
 
 		/* Get X */
 		if (zend_hash_find(sub_array, "x", sizeof("x"), (void**)&ppz_x) == FAILURE) {
+			efree(coordinates);
 			coordinates = (PointInfo *)NULL;
 			*num_elements = 0;
 			return coordinates;
 		}
 
 		if(Z_TYPE_PP(ppz_x) != IS_DOUBLE && Z_TYPE_PP(ppz_x) != IS_LONG) {
+			efree(coordinates);
 			coordinates = (PointInfo *)NULL;
 			*num_elements = 0;
 			return coordinates;
@@ -357,12 +391,14 @@ void *get_pointinfo_array(zval *coordinate_array, int *num_elements TSRMLS_DC)
 
 		/* Get Y */
 		if (zend_hash_find(sub_array, "y", sizeof("y"), (void**)&ppz_y) == FAILURE) {
+			efree(coordinates);
 			coordinates = (PointInfo *)NULL;
 			*num_elements = 0;
 			return coordinates;
 		}
 
 		if(Z_TYPE_PP(ppz_y) != IS_DOUBLE && Z_TYPE_PP(ppz_y) != IS_LONG) {
+			efree(coordinates);
 			coordinates = (PointInfo *)NULL;
 			*num_elements = 0;
 			return coordinates;
@@ -395,11 +431,9 @@ void count_pixeliterator_rows(php_imagickpixeliterator_object *internpix TSRMLS_
 	(void) PixelResetIterator(internpix->pixel_iterator);
 
 	while ((row = (PixelWand **)PixelGetNextIteratorRow(internpix->pixel_iterator, &tmp))) {
-		
 		if (row == (PixelWand **)NULL) {
 			break;
 		}
-
 		rows++;
 	}
 	internpix->rows = rows;
@@ -424,118 +458,88 @@ char *get_pseudo_filename(char *pseudo_string TSRMLS_DC)
 /* type 1 = writeimage, type 2 = writeimages */
 int write_image_from_filename(php_imagick_object *intern, char *filename, zend_bool adjoin, int type TSRMLS_DC)
 {
-	int error = IMAGICK_READ_WRITE_NO_ERROR, occurences = 0;
+	int pos = 0, occurances = 0, add_format = 0, error = IMAGICK_READ_WRITE_NO_ERROR;
+	char *buffer, *filepath, *final_filename, c;
 	MagickBooleanType status;
-	char *absolute, *buffer, *format, *tmp, *dup;
+
+	occurances = count_occurences_of(':', filename TSRMLS_CC);
 
 #if defined(PHP_WIN32)
-	if (count_occurences_of(':', filename TSRMLS_CC) == 2) {
-		occurences = 1;
-	} else {
-		occurences = 0;
+	/* windows can have things like png:C:\test.png */
+	if (occurances > 0) {
+		occurances -= 1;
 	}
-#else
-	occurences = count_occurences_of(':', filename TSRMLS_CC);
 #endif
-	switch (occurences) {
+
+	if (strlen(filename) > MAXPATHLEN) {
+		return IMAGICK_READ_WRITE_FILENAME_TOO_LONG;
+	}
+
+	/* Assume format in the filename. For example: png:hello.jpg */
+	if (occurances > 0) {
+		buffer = filename;
 		
-		case 0:
-
-			if (strlen(filename) > MAXPATHLEN) {
-				return IMAGICK_READ_WRITE_FILENAME_TOO_LONG;
-			}
-
-			absolute = expand_filepath(filename, NULL TSRMLS_CC);
-			IMAGICK_SAFE_MODE_CHECK(absolute, error);
-
-			if (error != IMAGICK_READ_WRITE_NO_ERROR) {
-				efree(absolute);
-				return error;
-			}
-			
-			error = check_write_access(absolute TSRMLS_CC);
-			
-			if (error != IMAGICK_READ_WRITE_NO_ERROR) {
-				efree(absolute);
-				return error;
-			}	
-	
-		break;
-
-		case 1:
-			/* Duplicate the filename */
-			dup = estrdup(filename);
-
-			if(!dup) {
-				return IMAGICK_READ_WRITE_UNDERLYING_LIBRARY;
-			}
-
-			format = strtok(dup, ":");
-			buffer = strtok(NULL, ":");
-
-			if(buffer == NULL) {
-				efree(dup);
-				return IMAGICK_READ_WRITE_UNDERLYING_LIBRARY;
-			}
-
-			if (strlen(buffer) > MAXPATHLEN) {
-				efree(dup);
-				return IMAGICK_READ_WRITE_FILENAME_TOO_LONG;
-			}
-
-			/* Safe mode checking */
-			tmp = expand_filepath(buffer, NULL TSRMLS_CC);
-			IMAGICK_SAFE_MODE_CHECK(tmp, error);
-
-			if (error != IMAGICK_READ_WRITE_NO_ERROR) {
-				efree(dup);
-				efree(tmp);
-				return error;
-			}
-      
-			error = check_write_access(tmp TSRMLS_CC);
-			
-			if (error != IMAGICK_READ_WRITE_NO_ERROR) {
-				efree(dup);
-				efree(tmp);
-				return error;
-			}      
-
-			/* Allocate space */
-			absolute = (char *)emalloc(strlen(format) + strlen(tmp) + 2);
-			memset(absolute, '\0', strlen(format) + strlen(tmp) + 2);
-
-			/* The final filename */
-			strncat(absolute, format, strlen(format));
-			strncat(absolute, ":", 1);
-			strncat(absolute, tmp, strlen(tmp));
-
-			efree(dup);
-			efree(tmp);   
-			/* absolute now contains the path */
-
-			if (error != IMAGICK_READ_WRITE_NO_ERROR) {
-				efree(absolute);
-				return error;
-			}
-
-		break;
-
-		default:
+		/* Strip everything before the first : */
+		while (((c = *(buffer++)) != '\0') && c != ':') { pos++; };
+		
+		/* This can happen with filename like "png:" */
+		if (buffer == '\0' || strlen(buffer) == 0) {
 			return IMAGICK_READ_WRITE_UNDERLYING_LIBRARY;
-		break;
-	}
-
-	if (type == 1) {
-		status = MagickWriteImage(intern->magick_wand, absolute);
+		}
+		
+		/* Absolute path to the file */
+		filepath = expand_filepath(buffer, NULL TSRMLS_CC);
+	
+		/* Indicate that the final format string will need the format */
+		add_format = 1;
 	} else {
-		status = MagickWriteImages(intern->magick_wand, absolute, adjoin);
+		filepath = expand_filepath(filename, NULL TSRMLS_CC);
+		add_format = 0;
+	}
+	
+	/* Check safe mode and open basedir */
+	IMAGICK_SAFE_MODE_CHECK(filepath, error);
+	if (error != IMAGICK_READ_WRITE_NO_ERROR) {
+		efree(filepath);
+		return error;
 	}
-	efree(absolute);
 
+	/* Bypass a bug in imagemagick. write failure causes a segfault*/
+	error = check_write_access(filepath TSRMLS_CC);
+	if (error != IMAGICK_READ_WRITE_NO_ERROR) {
+		efree(filepath);
+		return error;
+	}
+
+	/* Add the format to the string ? */
+	if (add_format) {
+		char *format = NULL, *tmp_filepath = estrdup(filepath);
+		efree(filepath);
+		filepath = NULL;
+		
+		format = emalloc(pos+1);
+		format[0] = '\0';
+		strncat(format, filename, pos);
+		
+		spprintf(&filepath, 0, "%s:%s", format, tmp_filepath);
+		efree(tmp_filepath);
+		efree(format);
+	}
+
+	/* Write image or write images */
+	if (type == 1) {
+		status = MagickWriteImage(intern->magick_wand, filepath);
+	} else {
+		status = MagickWriteImages(intern->magick_wand, filepath, adjoin);
+	}
+	efree(filepath);
+
+	/* Write succeded ? */
 	if (status == MagickFalse) {
 		return IMAGICK_READ_WRITE_UNDERLYING_LIBRARY;
 	}
+	
+	/* All went well it seems */
 	return IMAGICK_READ_WRITE_NO_ERROR;
 }
 
@@ -721,6 +725,9 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_BLACK", BlackChannel);
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_INDEX", IndexChannel);
 	IMAGICK_REGISTER_CONST_LONG("CHANNEL_ALL", AllChannels);
+#ifdef DefaultChannels
+	IMAGICK_REGISTER_CONST_LONG("CHANNEL_DEFAULT", DefaultChannels);
+#endif	
 	IMAGICK_REGISTER_CONST_LONG("METRIC_UNDEFINED", UndefinedMetric);
 	IMAGICK_REGISTER_CONST_LONG("METRIC_MEANABSOLUTEERROR", MeanAbsoluteErrorMetric);
 	IMAGICK_REGISTER_CONST_LONG("METRIC_MEANSQUAREERROR", MeanSquaredErrorMetric);
@@ -839,7 +846,7 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("PATHUNITS_USERSPACE", UserSpace);
 	IMAGICK_REGISTER_CONST_LONG("PATHUNITS_USERSPACEONUSE", UserSpaceOnUse);
 	IMAGICK_REGISTER_CONST_LONG("PATHUNITS_OBJECTBOUNDINGBOX", ObjectBoundingBox);
-	IMAGICK_REGISTER_CONST_LONG("LINECAP_UNDEFINED", UndefinedCap)
+	IMAGICK_REGISTER_CONST_LONG("LINECAP_UNDEFINED", UndefinedCap);
 	IMAGICK_REGISTER_CONST_LONG("LINECAP_BUTT", ButtCap);
 	IMAGICK_REGISTER_CONST_LONG("LINECAP_ROUND", RoundCap);
 	IMAGICK_REGISTER_CONST_LONG("LINECAP_SQUARE", SquareCap);
@@ -907,6 +914,15 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_PERSPECTIVEPROJECTION", PerspectiveProjectionDistortion);
 	IMAGICK_REGISTER_CONST_LONG("DISTORTION_SCALEROTATETRANSLATE", ScaleRotateTranslateDistortion);
 #endif
+#if MagickLibVersion > 0x645
+	IMAGICK_REGISTER_CONST_LONG("DISTORTION_POLYNOMIAL", PolynomialDistortion);
+	IMAGICK_REGISTER_CONST_LONG("DISTORTION_POLAR", PolarDistortion);
+	IMAGICK_REGISTER_CONST_LONG("DISTORTION_DEPOLAR", DePolarDistortion);
+	IMAGICK_REGISTER_CONST_LONG("DISTORTION_BARREL", BarrelDistortion);
+	IMAGICK_REGISTER_CONST_LONG("DISTORTION_BARRELINVERSE", BarrelInverseDistortion);
+	IMAGICK_REGISTER_CONST_LONG("DISTORTION_SHEPARDS", ShepardsDistortion);
+	IMAGICK_REGISTER_CONST_LONG("DISTORTION_SENTINEL", SentinelDistortion);
+#endif
 #ifdef HAVE_IMAGEMAGICK6364ORLATER
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_MERGE", MergeLayer);
 	IMAGICK_REGISTER_CONST_LONG("LAYERMETHOD_FLATTEN", FlattenLayer);
@@ -918,4 +934,13 @@ void initialize_imagick_constants()
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_RESET", ResetAlphaChannel);
 	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_SET", SetAlphaChannel);
 #endif
+#if MagickLibVersion > 0x645
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_UNDEFINED", UndefinedAlphaChannel);
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_COPY", CopyAlphaChannel);
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_DEACTIVATE", DeactivateAlphaChannel);
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_EXTRACT", ExtractAlphaChannel);
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_OPAQUE", OpaqueAlphaChannel);
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_SHAPE", ShapeAlphaChannel);
+	IMAGICK_REGISTER_CONST_LONG("ALPHACHANNEL_TRANSPARENT", TransparentAlphaChannel);	
+#endif
 }
