@@ -23,8 +23,8 @@
 
 /* Define Extension Properties */
 #define PHP_IMAGICK_EXTNAME    "imagick"
-#define PHP_IMAGICK_VERSION    "2.2.1"
-#define PHP_IMAGICK_EXTNUM     20201
+#define PHP_IMAGICK_VERSION    "2.2.2"
+#define PHP_IMAGICK_EXTNUM     20202
 
 /* Import configure options
    when building outside of
@@ -52,6 +52,9 @@
 #include <wand/magick-wand.h>
 #endif
 
+#ifndef DefaultChannels
+# define DefaultChannels (AllChannels &~ OpacityChannel)
+#endif
 
 /* Some extra headers */
 #include "Zend/zend_exceptions.h"
